@@ -68,9 +68,10 @@ This module provides a low-level scheduling mechanism, that be used to
 create cron-like schedules, the specifications can be provided as cron
 expression strings, lists of integer values or L<Junctions> of values.
 
-There is a single class method C<every> that takes a schedule specification
+There is a class method C<every> that takes a schedule specification
 and returns a L<Supply> that will emit a value (a L<DateTime>) on the
-schedule specified.
+schedule specified. There is also a class method C<at> that returns
+a Promise that will be kept at the specified time.
 
 This can be used to build custom scheduling services like C<cron> with
 additional code to read the specification from a file and arrange the
@@ -79,7 +80,7 @@ that may require to execute some code asynchronously periodically.
 
 There is a single base Supply that emits an event at a 1 second frequency
 in order to preserve the accuracy of the timings (in testing it may drift
-by up to 59 seconds on a long run due to system latency if it didn't 
+by up to 59 seconds on a long run due to system latency if it didn't
 match the seconds too,) so this may be a problem on a heavily loaded
 single core computer. The sub-minute granularity isn't provided for in
 the interface as it is easily achieved anyway with a basic supply, it
@@ -100,6 +101,27 @@ arguments. The arguments are infact passed directly to the constructor
 of C<Chronic::Description> (described below,) which is used to match
 the times that an event should occur.
 
+=head2 method at
+
+    multi method at(Int $i) returns Promise
+    multi method at(Instant:D $i) returns Promise
+    multi method at(Str:D $d) returns Promise
+    multi method at(DateTime $d) returns Promise
+
+This takes a datetime specification (either a L<DateTime> object,
+a L<Str> that can be parsed as a DateTime, an L<Instant> or an L<Int>
+representing the epoch seconds,) and returns a L<Promise> which will be
+kept with a DateTime when that time arrives.  If the supplied datetime
+specification resolves to the current time or is in the past the Promise
+will be returned Kept. The resolution of the comparison is at the second
+level (as with C<every> above,) and any fractional part in the presented
+DateTime will be truncated.
+
+If you want to do something at some time period from the current time
+then you may actually be better off using the C<in> method of L<Promise>
+which returns a Promise that will be kept a certain number of seconds
+in the future.
+
 =head2 method supply
 
    method supply() returns Supply
@@ -113,7 +135,7 @@ as in the synopsis for example.
 This is the class that is used to match the L<DateTime> against the frequency
 specification, all of the attributes are an C<any> L<Junction> and by default
 will match any allowed value for the period unit (hence the default is a one
-minute frequency.)  
+minute frequency.)
 
 The constructor allows the unit specifications to provided as Junctions,
 lists of integers, single integer values or strings in the form of cron
@@ -136,10 +158,11 @@ specifications for that unit:
      ``0,2,4,6,8,10,12,14,16,18,20,22'').  Steps are also permitted after an
      asterisk, so if you want to say ``every two hours'', just use ``*/2''.
 
-(From the L<FreeBSD manpage from crontab(5)|https://www.freebsd.org/cgi/man.cgi?crontab(5)>). For brevity only the names and ranges of the values are
-described.  The "name" forms for C<month> and C<day-of-week> are currently
-not supported because the localisation issues seemed more trouble than it's
-worth.
+(From the L<FreeBSD manpage from
+crontab(5)|https://www.freebsd.org/cgi/man.cgi?crontab(5)>). For brevity
+only the names and ranges of the values are described.  The "name" forms
+for C<month> and C<day-of-week> are currently not supported because the
+localisation issues seemed more trouble than it's worth.
 
 The allowed arguments to the constructor (and attributes of the class) are:
 
@@ -167,22 +190,22 @@ The day of the week (starting on Monday) in the range 1 .. 7
 
 =end pod
 
-class Chronic:ver<0.0.2>:auth<github:jonathanstowe> {
+class Chronic:ver<0.0.7>:auth<github:jonathanstowe>:api<1.0> {
     class Description {
 
-        sub expand-expression(Str $exp, Range $r) returns Array[Int] {
+        sub expand-expression(Str $exp, Range $r --> Array[Int] ) {
             my Int @values;
 
             my ($top, $divisor) = $exp.split('/');
 
             sub explode-item(Str $v) {
-	            if $v.contains('-') {
-		            my ( $min, $max ) = $v.split('-');
-		            my Range $r = $min.Int .. $max.Int;
-		            $r.list;
-	            }
-	            else {
-		            $v;
+                if $v.contains('-') {
+                    my ( $min, $max ) = $v.split('-');
+                    my Range $r = $min.Int .. $max.Int;
+                    $r.list;
+                }
+                else {
+                    $v;
                 }
             }
 
@@ -190,7 +213,7 @@ class Chronic:ver<0.0.2>:auth<github:jonathanstowe> {
                 @values = $r.list;
             }
             else {
-                @values = $top.split(',').flatmap(&explode-item).map(*.Int);
+                @values = $top.split(',').map(&explode-item).flat.map(*.Int);
             }
 
             if $divisor.defined {
@@ -340,7 +363,7 @@ class Chronic:ver<0.0.2>:auth<github:jonathanstowe> {
             self.bless(|%new-args);
         }
 
-        multi method ACCEPTS(DateTime $d) returns Bool {
+        multi method ACCEPTS(DateTime $d --> Bool ) {
             $d.second.Int   == 0        &&
             $d.minute       ~~ $!minute &&
             $d.hour         ~~ $!hour   &&
@@ -354,28 +377,53 @@ class Chronic:ver<0.0.2>:auth<github:jonathanstowe> {
     my Supply $supply;
 
     #| access the single supply creating it if necessary
-    method supply() {
+    method supply( --> Supply ) {
         if not $supply.defined {
             $supply = Supply.on-demand( -> $p {
-                Supply.interval(1).tap({ $p.emit(DateTime.now); });
+                Supply.interval(1).tap({ $p.emit(DateTime.now.truncated-to('second')); });
             });
         }
         $supply;
     }
 
     #| create a supply that fires on the time specification
-    method every(*%args) returns Supply {
+    method every(*%args --> Supply ) {
         my $description = Description.new(|%args);
         my $supply = self.supply.grep($description);
         $supply;
     }
 
+    multi method at(Int $i --> Promise ) {
+        self.at(DateTime.new($i));
+    }
+    multi method at(Instant:D $i --> Promise ) {
+        self.at(DateTime.new($i));
+    }
+    multi method at(Str:D $d --> Promise ) {
+        self.at(DateTime.new($d));
+    }
+    multi method at(DateTime $d --> Promise ) {
+        my $datetime = $d.truncated-to('second');
+        my $promise = Promise.new;
+        my $v = $promise.vow;
+
+        if $datetime <= DateTime.now.truncated-to('second') {
+            $v.keep($datetime);
+        }
+        else {
+            my $tap = self.supply.grep({ $_ == $datetime }).tap({
+                $tap.close;
+                $v.keep($_);
+            });
+        }
+        $promise;
+    }
 }
 
 use MONKEY-TYPING;
 
 augment class DateTime {
-    multi method ACCEPTS(Chronic::Description $d) returns Bool {
+    multi method ACCEPTS(Chronic::Description $d --> Bool ) {
         self.second.Int   == 0         &&
         self.minute       ~~ $d.minute &&
         self.hour         ~~ $d.hour   &&
@@ -383,7 +431,7 @@ augment class DateTime {
         self.month        ~~ $d.month  &&
         self.day-of-week  ~~ $d.day-of-week;
     }
-    
+
 }
 
 # vim: ft=perl6 expandtab sw=4
